@@ -1,7 +1,7 @@
 /* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* vvmplayer-preferences-window.c
  *
- * Copyright 2021 Chris Talbot <chris@talbothome.com>
+ * Copyright 2021-2022 Chris Talbot <chris@talbothome.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,6 +80,9 @@ struct _VvmPreferencesWindow
   int activate_vvm_button_clicked_timeout;
   int sync_vvm_button_clicked_timeout;
   int enable_vvm_switch_clicked_timeout;
+
+  //Watch ID
+  gulong vvm_settings_watch_id;
 };
 
 G_DEFINE_TYPE (VvmPreferencesWindow, vvm_preferences_window, HDY_TYPE_PREFERENCES_WINDOW)
@@ -216,8 +219,7 @@ vvm_preferences_window_populate (VvmPreferencesWindow *self)
     hdy_combo_row_set_selected_index (self->vvm_type_combo_row, VVMType_encoded);
 
   //Status
-  if(vvm_settings_get_mm_available (settings) &&
-     vvm_settings_get_service_available (settings)) {
+  if(vvm_settings_get_mm_available (settings)) {
     vvm_preferences_set_gtkimage_icon (self->vvmd_enabled_image, STATUS_GOOD);
   } else {
     vvm_preferences_set_gtkimage_icon (self->vvmd_enabled_image, STATUS_BAD);
@@ -235,6 +237,13 @@ vvm_preferences_window_populate (VvmPreferencesWindow *self)
 
 }
 
+static void
+update_settings_cb (VvmPreferencesWindow *self)
+{
+    vvm_preferences_refresh_settings ();
+    vvm_preferences_window_populate (self);
+}
+
 static gboolean
 refresh_settings_button_timeout (gpointer user_data)
 {
@@ -445,7 +454,9 @@ enable_vvm_switch_timeout(gpointer user_data)
   self->enable_vvm_switch_clicked_timeout = FALSE;
   vvm_preferences_set_hdyactionrow_text (self->enable_vvm_text, "Enable VVM");
 
-  refresh_settings_button_clicked_cb (self->refresh_settings_button, self);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->enable_vvm_switch), TRUE);
+
+  update_settings_cb (self);
 
   return FALSE;
 }
@@ -466,11 +477,12 @@ enable_vvm_switch_flipped_cb (GtkSwitch *widget,
     VvmVvmd *backend = vvm_vvmd_get_default ();
     self->enable_vvm_switch_clicked_timeout = TRUE;
 
+    gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
+
     if(vvm_settings_get_mm_available (settings) == FALSE) {
       g_warning("Cannot find vvmd!");
       vvm_preferences_set_hdyactionrow_text(self->enable_vvm_text, "Cannot find vvmd");
       g_timeout_add_seconds (2, enable_vvm_switch_timeout, self);
-      gtk_switch_set_active (self->enable_vvm_switch, FALSE);
       return FALSE;
     }
 
@@ -493,7 +505,6 @@ enable_vvm_switch_flipped_cb (GtkSwitch *widget,
 
   } else {
     g_debug ("enable_vvm_switch_clicked_cb() timed out!");
-    gtk_switch_set_active (self->enable_vvm_switch, VVMEnabled);
   }
     return FALSE;
 }
@@ -505,7 +516,7 @@ activate_vvm_button_timeout(gpointer user_data)
   self->activate_vvm_button_clicked_timeout = FALSE;
   vvm_preferences_set_button_content (self->activate_vvm_button, "Activate");
 
-  refresh_settings_button_clicked_cb (self->refresh_settings_button, self);
+  update_settings_cb (self);
   return FALSE;
 }
 
@@ -712,9 +723,22 @@ vvm_preferences_window_new (void)
   return g_object_new (HDY_TYPE_VVM_PREFERENCES_WINDOW, NULL);
 }
 
+static void
+preference_window_finalize (GObject *object)
+{
+  VvmPreferencesWindow *self = (VvmPreferencesWindow *)object;
+  VvmVvmd *backend = vvm_vvmd_get_default ();
+
+  g_clear_signal_handler (&self->vvm_settings_watch_id,
+                          backend);
+
+  G_OBJECT_CLASS (vvm_preferences_window_parent_class)->finalize (object);
+}
+
 static void
 vvm_preferences_window_class_init (VvmPreferencesWindowClass *klass)
 {
+  GObjectClass *object_class  = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -770,15 +794,22 @@ vvm_preferences_window_class_init (VvmPreferencesWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, VvmPreferencesWindow, vvm_type_combo_row);
 
   gtk_widget_class_bind_template_callback (widget_class, vvm_type_selected_cb);
+
+  object_class->finalize = preference_window_finalize;
 }
 
 static void
 vvm_preferences_window_init (VvmPreferencesWindow *self)
 {
+  VvmVvmd *backend = vvm_vvmd_get_default ();
+
   gtk_widget_init_template (GTK_WIDGET (self));
   lists_page_init (self);
   vvm_preferences_refresh_settings ();
   vvm_preferences_window_populate (self);
 
+  self->vvm_settings_watch_id = g_signal_connect_swapped (backend, "settings-updated",
+                                                          G_CALLBACK (update_settings_cb), self);
+
 
 }
