@@ -1,7 +1,7 @@
 /* -*- mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 /* vvmplayer-vvmd.c
  *
- * Copyright 2021 Chris Talbot
+ * Copyright 2021-2022 Chris Talbot
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,14 +25,6 @@
 
 #include "vvmplayer-vvmd.h"
 
-#define VVMD_SERVICE	                "org.kop316.vvm"
-#define VVMD_PATH	                "/org/kop316/vvm"
-#define VVMD_MODEMMANAGER_PATH          VVMD_PATH    "/modemmanager"
-#define VVMD_MANAGER_INTERFACE	        VVMD_SERVICE ".Manager"
-#define VVMD_SERVICE_INTERFACE	        VVMD_SERVICE ".Service"
-#define VVMD_MESSAGE_INTERFACE	        VVMD_SERVICE ".Message"
-#define VVMD_MODEMMANAGER_INTERFACE	VVMD_SERVICE ".ModemManager"
-
 struct _VvmVvmd
 {
   GObject     parent_instance;
@@ -58,6 +50,13 @@ struct _VvmVvmd
 
 G_DEFINE_TYPE (VvmVvmd, vvm_vvmd, G_TYPE_OBJECT)
 
+enum {
+  SETTINGS_UPDATED,
+  N_SIGNALS
+};
+
+static guint signals[N_SIGNALS];
+
 int
 vvmplayer_vvmd_encode_vvm_type (const char *vvm_type)
 {
@@ -578,6 +577,8 @@ vvmplayer_vvmd_get_service_cb (GObject      *service,
     } else {
       g_warning ("Failed to connect 'MessageAdded' signal");
     }
+    g_debug ("Emitting update settings signal");
+    g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
 
     g_dbus_proxy_call (self->service_proxy,
                        "GetMessages",
@@ -669,6 +670,10 @@ vvmplayer_vvmd_service_removed_cb (GDBusConnection *connection,
   g_debug ("%s", __func__);
   g_debug ("Service Removed g_variant: %s", g_variant_print (parameters, TRUE));
   vvmplayer_vvmd_remove_service (self);
+
+  g_debug ("Emitting update settings signal");
+  g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
+
 }
 
 static void
@@ -725,6 +730,9 @@ vvmplayer_vvmd_get_manager_cb (GObject      *manager,
       while ((service = g_variant_iter_next_value (&iter))) {
         vvmplayer_vvmd_connect_to_service (self, service);
       }
+    } else {
+      g_debug ("No Services, Sending emit to update settings");
+      g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
     }
   }
 }
@@ -738,16 +746,23 @@ vvmplayer_vvmd_provision_status_changed_cb (GDBusConnection *connection,
                                             GVariant        *parameters,
                                             gpointer         user_data)
 {
-  //VvmVvmd *self = user_data;
+  VvmVvmd *self = user_data;
   VvmSettings *settings = vvm_settings_get_default ();
   g_autoptr (GVariant) provisionstatus = NULL;
+  g_autofree char *Oldprovision = NULL;
   const char *provision;
 
   g_debug ("%s", __func__);
   g_variant_get (parameters, "(sv)", NULL, &provisionstatus);
   provision = g_variant_get_string (provisionstatus, NULL);
 
-  vvm_settings_set_vvm_provision_status (settings, provision);
+  Oldprovision = vvm_settings_get_vvm_provision_status (settings);
+
+  if (g_strcmp0(Oldprovision, provision)) {
+    vvm_settings_set_vvm_provision_status (settings, provision);
+    g_debug ("Emitting update settings signal");
+    g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
+  }
 }
 
 static void
@@ -759,21 +774,45 @@ vvmplayer_vvmd_provision_settings_changed_cb (GDBusConnection *connection,
                                             GVariant        *parameters,
                                             gpointer         user_data)
 {
-  //VvmVvmd *self = user_data;
+  VvmVvmd *self = user_data;
   VvmSettings *settings = vvm_settings_get_default ();
   g_autoptr (GError) error = NULL;
   g_autoptr (GVariant) provisionstatus = NULL;
+  g_autofree char *OldVVMType = NULL;
+  g_autofree char *OldVVMDestinationNumber = NULL;
+  g_autofree char *OldCarrierPrefix = NULL;
   const char     *VVMType = NULL;
   const char     *VVMDestinationNumber = NULL;
   const char     *CarrierPrefix = NULL;
+  gboolean settings_changed = FALSE;
 
 
   g_debug ("%s", __func__);
   g_variant_get (parameters, "(sss)", &VVMType, &VVMDestinationNumber, &CarrierPrefix);
 
-  vvm_settings_set_vvm_type (settings, VVMType);
-  vvm_settings_set_vvm_destination_number (settings, VVMDestinationNumber);
-  vvm_settings_set_vvm_carrier_prefix (settings, CarrierPrefix);
+  OldVVMType = vvm_settings_get_vvm_type (settings);
+  OldVVMDestinationNumber = vvm_settings_get_vvm_destination_number (settings);
+  OldCarrierPrefix = vvm_settings_get_vvm_carrier_prefix (settings);
+
+  if (g_strcmp0(OldVVMType, VVMType)) {
+    vvm_settings_set_vvm_type (settings, VVMType);
+    settings_changed = TRUE;
+  }
+
+  if (g_strcmp0(OldVVMDestinationNumber, VVMDestinationNumber)) {
+    vvm_settings_set_vvm_destination_number (settings, VVMDestinationNumber);
+    settings_changed = TRUE;
+  }
+
+  if (g_strcmp0(OldCarrierPrefix, CarrierPrefix)) {
+    vvm_settings_set_vvm_carrier_prefix (settings, CarrierPrefix);
+    settings_changed = TRUE;
+  }
+
+  if (settings_changed) {
+    g_debug ("Emitting update settings signal");
+    g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
+  }
 }
 
 int
@@ -951,6 +990,8 @@ vvmd_vanished_cb (GDBusConnection *connection,
   VvmSettings *settings = vvm_settings_get_default ();
 
   g_debug ("VVMD vanished");
+
+  vvm_settings_set_mailbox_active (settings, FALSE);
   vvm_settings_set_mm_available (settings, FALSE);
   vvm_settings_set_service_available (settings, FALSE);
 
@@ -976,6 +1017,8 @@ vvmd_vanished_cb (GDBusConnection *connection,
                                          self->vvmd_watch_id);
   }
 
+  g_debug ("Sending emit to update settings");
+  g_signal_emit (self, signals[SETTINGS_UPDATED], 0);
 }
 
 static void
@@ -1015,6 +1058,13 @@ vvm_vvmd_class_init (VvmVvmdClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  signals [SETTINGS_UPDATED] =
+    g_signal_new ("settings-updated",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
   object_class->constructed = vvm_vvmd_constructed;
   object_class->finalize = vvm_vvmd_finalize;
 }
